#include "gideros.h"
#include "lua.h"
#include "lauxlib.h"
#define LUA_LIB
#include <jni.h>
//this is for debugginh purpose
//and should be commented out before deployment
//you can log using
//__android_log_print(ANDROID_LOG_DEBUG, "tag", "Output String");
#include <android/log.h>

//some configurations of our plugin
static const char* pluginName = "googlelvl";
static const char* pluginVersion = "1";
static const char* javaClassName = "com/giderosmobile/android/plugins/googlelicensing/GoogleLVL";

//Store Java Environment reference
static JNIEnv *ENV;
//Store our main class, what we will use as plugin
static jclass cls;
//store modifyString method ID
static jmethodID jSetKey;
static jmethodID jCheckLicense;


static const char *LICENSE_ALLOWED = "licenseAllowed";
static const char *LICENSE_DENIED = "licenseDenied";
static const char *LICENSE_RETRY = "licenseRetry";

//set public key
static int setKey(lua_State *L)
{
	//if no Java Env, exit
	if(ENV == NULL) return 0;
	
	//check if there is a class
	if(!cls) return 0;
	
	//if we don't have method yet, try to retrieve it
	if(jSetKey == NULL)
	{
		/****************
		* 1. argument cls - reference to Java class, where we have this method
		* 2. argument name of the method to get
		* 3. argument what arguments does method accept and what it returns
		****************/
		jSetKey = ENV->GetStaticMethodID(cls, "setKey", "(Ljava/lang/String;)V");
		if(!jSetKey) return 0;
	}
	
	//get arguments from lua
	const char *publicKey = lua_tostring(L, 1);
	
	//call java method with parameters
	jstring jPublicKey = ENV->NewStringUTF(publicKey);
	ENV->CallStaticVoidMethod(cls, jSetKey, jPublicKey);
	ENV->DeleteLocalRef(jPublicKey);
	
	return 1;
}

//check license
static int checkLicense(lua_State *L)
{
	//if no Java Env, exit
	if(ENV == NULL) return 0;
	
	//check if there is a class
	if(!cls) return 0;
	
	//if we don't have method yet, try to retrieve it
	if(jCheckLicense == NULL)
	{
		/****************
		* 1. argument cls - reference to Java class, where we have this method
		* 2. argument name of the method to get
		* 3. argument what arguments does method accept and what it returns
		****************/
		jCheckLicense = ENV->GetStaticMethodID(cls, "checkLicense", "()V");
		if(!jCheckLicense) return 0;
	}
	
	//call java method with parameters
	ENV->CallStaticVoidMethod(cls, jCheckLicense);
	
	return 1;
}

//here we register all functions we could call from lua
//lua function name as key and C function as value
static const struct luaL_Reg funcs[] = {
  { "setKey",	setKey },
  { "checkLicense",	checkLicense },
  { NULL, NULL }//don't forget nulls at the end
};

//here we register all the C functions for lua
//so lua engine would know they exists
LUALIB_API int luaopen_plugin(lua_State *L)
{
  luaL_register(L, pluginName, funcs);
  return 1;
}

//here we do all our stuff needs to be done on initialization
static void g_initializePlugin(lua_State *L)
{
	//get java environment reference
	ENV = g_getJNIEnv();
	
	jclass class_local = ENV->FindClass(javaClassName);
	if(class_local)
	{
		cls = static_cast<jclass>(ENV->NewGlobalRef(class_local)); // Create the global reference
		ENV->DeleteLocalRef(class_local); // Delete the local reference
	}
	
	//get global package object
	lua_getglobal(L, "package");
	lua_getfield(L, -1, "preload");
	
	//put our plugin name inside with a callback to
	//registering C functions
	lua_pushcfunction(L, luaopen_plugin);
	lua_setfield(L, -2, pluginName);

	lua_pop(L, 2);
	
	lua_getglobal(L, "Event");
	lua_pushstring(L, LICENSE_ALLOWED);
	lua_setfield(L, -2, "LICENSE_ALLOWED");
	lua_pushstring(L, LICENSE_DENIED);
	lua_setfield(L, -2, "LICENSE_DENIED");
	lua_pushstring(L, LICENSE_RETRY);
	lua_setfield(L, -2, "LICENSE_RETRY");
	lua_pop(L, 1);
}

//and here we free everything we need to free
static void g_deinitializePlugin(lua_State *L)
{
	ENV->DeleteGlobalRef(cls);
}

//register our plugin with Gideros lib
REGISTER_PLUGIN(pluginName, pluginVersion)